@@ -1029,10 +1029,7 @@ xplr.config.modes.builtin.default = {
         help = "show hidden",
         messages = {
           {
-            ToggleNodeFilter = {
-              filter = "RelativePathDoesNotStartWith",
-              input = ".",
-            },
+            ToggleNodeFilter = { filter = "RelativePathDoesNotStartWith", input = "." },
           },
           "ExplorePwdAsync",
         },
@@ -1157,9 +1154,9 @@ xplr.config.modes.builtin.default = {
           "PopMode",
           { SwitchModeBuiltin = "rename" },
           {
-            BashExecSilently = [===[
+            BashExecSilently0 = [===[
               NAME=$(basename "${XPLR_FOCUS_PATH:?}")
-              echo SetInputBuffer: "'"${NAME:?}"'" >> "${XPLR_PIPE_MSG_IN:?}"
+              "$XPLR" -m 'SetInputBuffer: %q' "${NAME:?}"
             ]===],
           },
         },
@@ -1170,9 +1167,9 @@ xplr.config.modes.builtin.default = {
           "PopMode",
           { SwitchModeBuiltin = "duplicate_as" },
           {
-            BashExecSilently = [===[
+            BashExecSilently0 = [===[
               NAME=$(basename "${XPLR_FOCUS_PATH:?}")
-              echo SetInputBuffer: "'"${NAME:?}"'" >> "${XPLR_PIPE_MSG_IN:?}"
+              "$XPLR" -m 'SetInputBuffer: %q' "${NAME:?}"
             ]===],
           },
         },
@@ -1207,8 +1204,8 @@ xplr.config.modes.builtin.default = {
         help = "go home",
         messages = {
           {
-            BashExecSilently = [===[
-              echo ChangeDirectory: "'"${HOME:?}"'" >> "${XPLR_PIPE_MSG_IN:?}"
+            BashExecSilently0 = [===[
+              "$XPLR" -m 'ChangeDirectory: %q' "${HOME:?}"
             ]===],
           },
         },
@@ -1251,27 +1248,20 @@ xplr.config.modes.builtin.default = {
 
 xplr.config.modes.builtin.default.key_bindings.on_key["tab"] =
   xplr.config.modes.builtin.default.key_bindings.on_key["ctrl-i"]
-
 xplr.config.modes.builtin.default.key_bindings.on_key["v"] =
-  xplr.config.modes.builtin.default.key_bindings.on_key.space
-
+  xplr.config.modes.builtin.default.key_bindings.on_key["space"]
 xplr.config.modes.builtin.default.key_bindings.on_key["V"] =
   xplr.config.modes.builtin.default.key_bindings.on_key["ctrl-a"]
-
 xplr.config.modes.builtin.default.key_bindings.on_key["/"] =
   xplr.config.modes.builtin.default.key_bindings.on_key["ctrl-f"]
-
 xplr.config.modes.builtin.default.key_bindings.on_key["h"] =
-  xplr.config.modes.builtin.default.key_bindings.on_key.left
-
+  xplr.config.modes.builtin.default.key_bindings.on_key["left"]
 xplr.config.modes.builtin.default.key_bindings.on_key["j"] =
-  xplr.config.modes.builtin.default.key_bindings.on_key.down
-
+  xplr.config.modes.builtin.default.key_bindings.on_key["down"]
 xplr.config.modes.builtin.default.key_bindings.on_key["k"] =
-  xplr.config.modes.builtin.default.key_bindings.on_key.up
-
+  xplr.config.modes.builtin.default.key_bindings.on_key["up"]
 xplr.config.modes.builtin.default.key_bindings.on_key["l"] =
-  xplr.config.modes.builtin.default.key_bindings.on_key.right
+  xplr.config.modes.builtin.default.key_bindings.on_key["right"]
 
 -- The builtin debug error mode.
 --
@@ -1320,7 +1310,7 @@ xplr.config.modes.builtin.debug_error = {
         messages = {
           {
             BashExec = [===[
-              ${EDITOR:-vi} "${XPLR_PIPE_LOGS_OUT:?}"
+              cat "${XPLR_PIPE_LOGS_OUT:?}" | ${EDITOR:-vi} -
             ]===],
           },
         },
@@ -1380,11 +1370,15 @@ xplr.config.modes.builtin.go_to_path = {
         help = "submit",
         messages = {
           {
-            BashExecSilently = [===[
-              if [ -d "$XPLR_INPUT_BUFFER" ]; then
-                echo ChangeDirectory: "'"$XPLR_INPUT_BUFFER"'" >> "${XPLR_PIPE_MSG_IN:?}"
-              elif [ -e "$XPLR_INPUT_BUFFER" ]; then
-                echo FocusPath: "'"$XPLR_INPUT_BUFFER"'" >> "${XPLR_PIPE_MSG_IN:?}"
+            BashExecSilently0 = [===[
+              PTH=${XPLR_INPUT_BUFFER}
+              PTH_ESC=$(printf %q "$PTH")
+              if [ -d "$PTH" ]; then
+                "$XPLR" -m 'ChangeDirectory: %q' "$PTH"
+              elif [ -e "$PTH" ]; then
+                "$XPLR" -m 'FocusPath: %q' "$PTH"
+              else
+                "$XPLR" -m 'LogError: %q' "could not find $PTH_ESC"
               fi
             ]===],
           },
@@ -1411,22 +1405,24 @@ xplr.config.modes.builtin.go_to_path = {
 -- Type: [Mode](https://xplr.dev/en/mode)
 xplr.config.modes.builtin.selection_ops = {
   name = "selection ops",
+  layout = "HelpMenu",
   key_bindings = {
     on_key = {
       ["c"] = {
         help = "copy here",
         messages = {
           {
-            BashExec = [===[
-              (while IFS= read -r line; do
-              if cp -vr -- "${line:?}" ./; then
-                echo LogSuccess: $line copied to $PWD >> "${XPLR_PIPE_MSG_IN:?}"
-              else
-                echo LogError: Failed to copy $line to $PWD >> "${XPLR_PIPE_MSG_IN:?}"
-              fi
+            BashExec0 = [===[
+              (while IFS= read -r -d '' PTH; do
+                PTH_ESC=$(printf %q "$PTH")
+                if cp -vr -- "${PTH:?}" ./; then
+                  "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC copied to ."
+                else
+                  "$XPLR" -m 'LogError: %q' "Failed to copy $PTH_ESC to ."
+                fi
               done < "${XPLR_PIPE_SELECTION_OUT:?}")
-              echo ExplorePwdAsync >> "${XPLR_PIPE_MSG_IN:?}"
-              echo ClearSelection >> "${XPLR_PIPE_MSG_IN:?}"
+              "$XPLR" -m ExplorePwdAsync
+              "$XPLR" -m ClearSelection
               read -p "[enter to continue]"
             ]===],
           },
@@ -1437,15 +1433,16 @@ xplr.config.modes.builtin.selection_ops = {
         help = "move here",
         messages = {
           {
-            BashExec = [===[
-              (while IFS= read -r line; do
-              if mv -v -- "${line:?}" ./; then
-                echo LogSuccess: $line moved to $PWD >> "${XPLR_PIPE_MSG_IN:?}"
-              else
-                echo LogError: Failed to move $line to $PWD >> "${XPLR_PIPE_MSG_IN:?}"
-              fi
+            BashExec0 = [===[
+              (while IFS= read -r -d '' PTH; do
+                PTH_ESC=$(printf %q "$PTH")
+                if mv -v -- "${PTH:?}" ./; then
+                  "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC moved to ."
+                else
+                  "$XPLR" -m 'LogError: %q' "Failed to move $PTH_ESC to ."
+                fi
               done < "${XPLR_PIPE_SELECTION_OUT:?}")
-              echo ExplorePwdAsync >> "${XPLR_PIPE_MSG_IN:?}"
+              "$XPLR" -m ExplorePwdAsync
               read -p "[enter to continue]"
             ]===],
           },
@@ -1459,30 +1456,6 @@ xplr.config.modes.builtin.selection_ops = {
           "PopMode",
         },
       },
-      ["x"] = {
-        help = "open in gui",
-        messages = {
-          {
-            BashExecSilently = [===[
-              if [ -z "$OPENER" ]; then
-                if command -v xdg-open; then
-                  OPENER=xdg-open
-                  elif command -v open; then
-                  OPENER=open
-                else
-                  echo 'LogError: $OPENER not found' >> "${XPLR_PIPE_MSG_IN:?}"
-                  exit 1
-                fi
-              fi
-              (while IFS= read -r line; do
-              $OPENER "${line:?}" > /dev/null 2>&1
-              done < "${XPLR_PIPE_RESULT_OUT:?}")
-            ]===],
-          },
-          "ClearScreen",
-          "PopMode",
-        },
-      },
     },
   },
 }
@@ -1492,6 +1465,7 @@ xplr.config.modes.builtin.selection_ops = {
 -- Type: [Mode](https://xplr.dev/en/mode)
 xplr.config.modes.builtin.create = {
   name = "create",
+  layout = "HelpMenu",
   key_bindings = {
     on_key = {
       ["d"] = {
@@ -1532,16 +1506,17 @@ xplr.config.modes.builtin.create_directory = {
         help = "submit",
         messages = {
           {
-            BashExecSilently = [===[
+            BashExecSilently0 = [===[
               PTH="$XPLR_INPUT_BUFFER"
-              if [ "${PTH}" ]; then
-                mkdir -p -- "${PTH:?}" \
-                && echo "SetInputBuffer: ''" >> "${XPLR_PIPE_MSG_IN:?}" \
-                && echo ExplorePwd >> "${XPLR_PIPE_MSG_IN:?}" \
-                && echo LogSuccess: $PTH created >> "${XPLR_PIPE_MSG_IN:?}" \
-                && echo FocusPath: "'"$PTH"'" >> "${XPLR_PIPE_MSG_IN:?}"
+              PTH_ESC=$(printf %q "$PTH")
+              if [ "$PTH" ]; then
+                mkdir -p -- "$PTH" \
+                && "$XPLR" -m 'SetInputBuffer: ""' \
+                && "$XPLR" -m ExplorePwd \
+                && "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC created" \
+                && "$XPLR" -m 'FocusPath: %q' "$PTH"
               else
-                echo PopMode >> "${XPLR_PIPE_MSG_IN:?}"
+                "$XPLR" -m PopMode
               fi
             ]===],
           },
@@ -1574,17 +1549,18 @@ xplr.config.modes.builtin.create_file = {
         help = "submit",
         messages = {
           {
-            BashExecSilently = [===[
+            BashExecSilently0 = [===[
               PTH="$XPLR_INPUT_BUFFER"
+              PTH_ESC=$(printf %q "$PTH")
               if [ "$PTH" ]; then
-                mkdir -p -- "$(dirname $PTH)" \
+                mkdir -p -- "$(dirname $(realpath $PTH))" \
                 && touch -- "$PTH" \
-                && echo "SetInputBuffer: ''" >> "${XPLR_PIPE_MSG_IN:?}" \
-                && echo LogSuccess: $PTH created >> "${XPLR_PIPE_MSG_IN:?}" \
-                && echo ExplorePwd >> "${XPLR_PIPE_MSG_IN:?}" \
-                && echo FocusPath: "'"$PTH"'" >> "${XPLR_PIPE_MSG_IN:?}"
+                && "$XPLR" -m 'SetInputBuffer: ""' \
+                && "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC created" \
+                && "$XPLR" -m 'ExplorePwd' \
+                && "$XPLR" -m 'FocusPath: %q' "$PTH"
               else
-                echo PopMode >> "${XPLR_PIPE_MSG_IN:?}"
+                "$XPLR" -m PopMode
               fi
             ]===],
           },
@@ -1644,15 +1620,16 @@ xplr.config.modes.builtin.number = {
 }
 
 xplr.config.modes.builtin.number.key_bindings.on_key["j"] =
-  xplr.config.modes.builtin.number.key_bindings.on_key.down
+  xplr.config.modes.builtin.number.key_bindings.on_key["down"]
 xplr.config.modes.builtin.number.key_bindings.on_key["k"] =
-  xplr.config.modes.builtin.number.key_bindings.on_key.up
+  xplr.config.modes.builtin.number.key_bindings.on_key["up"]
 
 -- The builtin go to mode.
 --
 -- Type: [Mode](https://xplr.dev/en/mode)
 xplr.config.modes.builtin.go_to = {
   name = "go to",
+  layout = "HelpMenu",
   key_bindings = {
     on_key = {
       ["f"] = {
@@ -1677,22 +1654,35 @@ xplr.config.modes.builtin.go_to = {
           { SetInputBuffer = "" },
         },
       },
+      ["i"] = {
+        help = "initial $PWD",
+        messages = {
+          "PopMode",
+          {
+            BashExecSilently0 = [===[
+              "$XPLR" -m 'ChangeDirectory: %q' "${XPLR_INITIAL_PWD:?}"
+            ]===],
+          },
+        },
+      },
       ["x"] = {
         help = "open in gui",
         messages = {
           {
-            BashExecSilently = [===[
+            BashExecSilently0 = [===[
               if [ -z "$OPENER" ]; then
                 if command -v xdg-open; then
                   OPENER=xdg-open
                   elif command -v open; then
                   OPENER=open
                 else
-                  echo 'LogError: $OPENER not found' >> "${XPLR_PIPE_MSG_IN:?}"
+                  "$XPLR" -m 'LogError: "$OPENER not found"'
                   exit 1
                 fi
               fi
-              $OPENER "${XPLR_FOCUS_PATH:?}" > /dev/null 2>&1
+              (while IFS= read -r -d '' PTH; do
+                $OPENER "${PTH:?}" > /dev/null 2>&1
+              done < "${XPLR_PIPE_RESULT_OUT:?}")
             ]===],
           },
           "ClearScreen",
@@ -1720,16 +1710,18 @@ xplr.config.modes.builtin.rename = {
         help = "submit",
         messages = {
           {
-            BashExecSilently = [===[
+            BashExecSilently0 = [===[
               SRC="${XPLR_FOCUS_PATH:?}"
+              SRC_ESC=$(printf %q "$SRC")
               TARGET="${XPLR_INPUT_BUFFER:?}"
+              TARGET_ESC=$(printf %q "$TARGET")
               if [ -e "${TARGET:?}" ]; then
-                echo LogError: $TARGET already exists >> "${XPLR_PIPE_MSG_IN:?}"
+                "$XPLR" -m 'LogError: %q' "$TARGET_ESC already exists"
               else
                 mv -- "${SRC:?}" "${TARGET:?}" \
-                  && echo ExplorePwd >> "${XPLR_PIPE_MSG_IN:?}" \
-                  && echo FocusPath: "'"$TARGET"'" >> "${XPLR_PIPE_MSG_IN:?}" \
-                  && echo LogSuccess: $SRC renamed to $TARGET >> "${XPLR_PIPE_MSG_IN:?}"
+                  && "$XPLR" -m ExplorePwd \
+                  && "$XPLR" -m 'FocusPath: %q' "$TARGET" \
+                  && "$XPLR" -m 'LogSuccess: %q' "$SRC_ESC renamed to $TARGET_ESC"
               fi
             ]===],
           },
@@ -1762,16 +1754,18 @@ xplr.config.modes.builtin.duplicate_as = {
         help = "submit",
         messages = {
           {
-            BashExecSilently = [===[
+            BashExecSilently0 = [===[
               SRC="${XPLR_FOCUS_PATH:?}"
+              SRC_ESC=$(printf %q "$SRC")
               TARGET="${XPLR_INPUT_BUFFER:?}"
+              TARGET_ESC=$(printf %q "$TARGET")
               if [ -e "${TARGET:?}" ]; then
-                echo LogError: $TARGET already exists >> "${XPLR_PIPE_MSG_IN:?}"
+                "$XPLR" -m 'LogError: %q' "$TARGET_ESC already exists"
               else
                 cp -r -- "${SRC:?}" "${TARGET:?}" \
-                  && echo ExplorePwd >> "${XPLR_PIPE_MSG_IN:?}" \
-                  && echo FocusPath: "'"$TARGET"'" >> "${XPLR_PIPE_MSG_IN:?}" \
-                  && echo LogSuccess: $SRC duplicated as $TARGET >> "${XPLR_PIPE_MSG_IN:?}"
+                  && "$XPLR" -m ExplorePwd \
+                  && "$XPLR" -m 'FocusPath: %q' "$TARGET_ESC" \
+                  && "$XPLR" -m 'LogSuccess: %q' "$SRC_ESC duplicated as $TARGET_ESC"
               fi
             ]===],
           },
@@ -1792,21 +1786,23 @@ xplr.config.modes.builtin.duplicate_as = {
 -- Type: [Mode](https://xplr.dev/en/mode)
 xplr.config.modes.builtin.delete = {
   name = "delete",
+  layout = "HelpMenu",
   key_bindings = {
     on_key = {
       ["D"] = {
         help = "force delete",
         messages = {
           {
-            BashExec = [===[
-              (while IFS= read -r line; do
-              if rm -rfv -- "${line:?}"; then
-                echo LogSuccess: $line deleted >> "${XPLR_PIPE_MSG_IN:?}"
-              else
-                echo LogError: Failed to delete $line >> "${XPLR_PIPE_MSG_IN:?}"
-              fi
+            BashExec0 = [===[
+              (while IFS= read -r -d '' PTH; do
+                PTH_ESC=$(printf %q "$PTH")
+                if rm -rfv -- "${PTH:?}"; then
+                  "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC deleted"
+                else
+                  "$XPLR" -m 'LogError: %q' "Failed to delete $PTH_ESC"
+                fi
               done < "${XPLR_PIPE_RESULT_OUT:?}")
-              echo ExplorePwdAsync >> "${XPLR_PIPE_MSG_IN:?}"
+              "$XPLR" -m ExplorePwdAsync
               read -p "[enter to continue]"
             ]===],
           },
@@ -1817,23 +1813,24 @@ xplr.config.modes.builtin.delete = {
         help = "delete",
         messages = {
           {
-            BashExec = [===[
-              (while IFS= read -r line; do
-              if [ -d "$line" ] && [ ! -L "$line" ]; then
-                if rmdir -v -- "${line:?}"; then
-                  echo LogSuccess: $line deleted >> "${XPLR_PIPE_MSG_IN:?}"
+            BashExec0 = [===[
+              (while IFS= read -r -d '' PTH; do
+                PTH_ESC=$(printf %q "$PTH")
+                if [ -d "$PTH" ] && [ ! -L "$PTH" ]; then
+                  if rmdir -v -- "${PTH:?}"; then
+                    "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC deleted"
+                  else
+                    "$XPLR" -m 'LogError: %q' "Failed to delete $PTH_ESC"
+                  fi
                 else
-                  echo LogError: Failed to delete $line >> "${XPLR_PIPE_MSG_IN:?}"
+                  if rm -v -- "${PTH:?}"; then
+                    "$XPLR" -m 'LogSuccess: %q' "$PTH_ESC deleted"
+                  else
+                    "$XPLR" -m 'LogError: %q' "Failed to delete $PTH_ESC"
+                  fi
                 fi
-              else
-                if rm -v -- "${line:?}"; then
-                  echo LogSuccess: $line deleted >> "${XPLR_PIPE_MSG_IN:?}"
-                else
-                  echo LogError: Failed to delete $line >> "${XPLR_PIPE_MSG_IN:?}"
-                fi
-              fi
               done < "${XPLR_PIPE_RESULT_OUT:?}")
-              echo ExplorePwdAsync >> "${XPLR_PIPE_MSG_IN:?}"
+              "$XPLR" -m ExplorePwdAsync
               read -p "[enter to continue]"
             ]===],
           },
@@ -1849,14 +1846,15 @@ xplr.config.modes.builtin.delete = {
 -- Type: [Mode](https://xplr.dev/en/mode)
 xplr.config.modes.builtin.action = {
   name = "action to",
+  layout = "HelpMenu",
   key_bindings = {
     on_key = {
       ["!"] = {
         help = "shell",
         messages = {
-          { Call = { command = "bash", args = { "-i" } } },
-          "ExplorePwdAsync",
           "PopMode",
+          { Call0 = { command = os.getenv("SHELL") or "bash", args = { "-i" } } },
+          "ExplorePwdAsync",
         },
       },
       ["c"] = {
@@ -1870,7 +1868,7 @@ xplr.config.modes.builtin.action = {
         help = "open in editor",
         messages = {
           {
-            BashExec = [===[
+            BashExec0 = [===[
               ${EDITOR:-vi} "${XPLR_FOCUS_PATH:?}"
             ]===],
           },
@@ -1903,6 +1901,13 @@ xplr.config.modes.builtin.action = {
           "ToggleMouse",
         },
       },
+      ["v"] = {
+        help = "vroot",
+        messages = {
+          "PopMode",
+          { SwitchModeBuiltin = "vroot" },
+        },
+      },
       ["q"] = {
         help = "quit options",
         messages = {
@@ -1927,6 +1932,7 @@ xplr.config.modes.builtin.action = {
 -- Type: [Mode](https://xplr.dev/en/mode)
 xplr.config.modes.builtin.quit = {
   name = "quit",
+  layout = "HelpMenu",
   key_bindings = {
     on_key = {
       ["enter"] = {
@@ -2030,9 +2036,9 @@ xplr.config.modes.builtin.search = {
 }
 
 xplr.config.modes.builtin.search.key_bindings.on_key["ctrl-n"] =
-  xplr.config.modes.builtin.search.key_bindings.on_key.down
+  xplr.config.modes.builtin.search.key_bindings.on_key["down"]
 xplr.config.modes.builtin.search.key_bindings.on_key["ctrl-p"] =
-  xplr.config.modes.builtin.search.key_bindings.on_key.up
+  xplr.config.modes.builtin.search.key_bindings.on_key["up"]
 
 -- The builtin filter mode.
 --
@@ -2167,72 +2173,37 @@ xplr.config.modes.builtin.sort = {
       ["E"] = {
         help = "by canonical extension reverse",
         messages = {
-          {
-            AddNodeSorter = {
-              sorter = "ByCanonicalExtension",
-              reverse = true,
-            },
-          },
+          { AddNodeSorter = { sorter = "ByCanonicalExtension", reverse = true } },
           "ExplorePwdAsync",
         },
       },
       ["M"] = {
         help = "by canonical mime essence reverse",
         messages = {
-          {
-            AddNodeSorter = {
-              sorter = "ByCanonicalMimeEssence",
-              reverse = true,
-            },
-          },
+          { AddNodeSorter = { sorter = "ByCanonicalMimeEssence", reverse = true } },
           "ExplorePwdAsync",
         },
       },
       ["N"] = {
         help = "by node type reverse",
         messages = {
-          {
-            AddNodeSorter = {
-              sorter = "ByCanonicalIsDir",
-              reverse = true,
-            },
-          },
-          {
-            AddNodeSorter = {
-              sorter = "ByCanonicalIsFile",
-              reverse = true,
-            },
-          },
-          {
-            AddNodeSorter = {
-              sorter = "ByIsSymlink",
-              reverse = true,
-            },
-          },
+          { AddNodeSorter = { sorter = "ByCanonicalIsDir", reverse = true } },
+          { AddNodeSorter = { sorter = "ByCanonicalIsFile", reverse = true } },
+          { AddNodeSorter = { sorter = "ByIsSymlink", reverse = true } },
           "ExplorePwdAsync",
         },
       },
       ["R"] = {
         help = "by relative path reverse",
         messages = {
-          {
-            AddNodeSorter = {
-              sorter = "ByIRelativePath",
-              reverse = true,
-            },
-          },
+          { AddNodeSorter = { sorter = "ByIRelativePath", reverse = true } },
           "ExplorePwdAsync",
         },
       },
       ["S"] = {
         help = "by size reverse",
         messages = {
-          {
-            AddNodeSorter = {
-              sorter = "BySize",
-              reverse = true,
-            },
-          },
+          { AddNodeSorter = { sorter = "BySize", reverse = true } },
           "ExplorePwdAsync",
         },
       },
@@ -2260,12 +2231,7 @@ xplr.config.modes.builtin.sort = {
       ["e"] = {
         help = "by canonical extension",
         messages = {
-          {
-            AddNodeSorter = {
-              sorter = "ByCanonicalExtension",
-              reverse = false,
-            },
-          },
+          { AddNodeSorter = { sorter = "ByCanonicalExtension", reverse = false } },
           "ExplorePwdAsync",
         },
       },
@@ -2278,27 +2244,16 @@ xplr.config.modes.builtin.sort = {
       ["m"] = {
         help = "by canonical mime essence",
         messages = {
-          {
-            AddNodeSorter = {
-              sorter = "ByCanonicalMimeEssence",
-              reverse = false,
-            },
-          },
+          { AddNodeSorter = { sorter = "ByCanonicalMimeEssence", reverse = false } },
           "ExplorePwdAsync",
         },
       },
       ["n"] = {
         help = "by node type",
         messages = {
-          {
-            AddNodeSorter = { sorter = "ByCanonicalIsDir", reverse = false },
-          },
-          {
-            AddNodeSorter = { sorter = "ByCanonicalIsFile", reverse = false },
-          },
-          {
-            AddNodeSorter = { sorter = "ByIsSymlink", reverse = false },
-          },
+          { AddNodeSorter = { sorter = "ByCanonicalIsDir", reverse = false } },
+          { AddNodeSorter = { sorter = "ByCanonicalIsFile", reverse = false } },
+          { AddNodeSorter = { sorter = "ByIsSymlink", reverse = false } },
           "ExplorePwdAsync",
         },
       },
@@ -2357,6 +2312,7 @@ xplr.config.modes.builtin.sort = {
 -- Type: [Mode](https://xplr.dev/en/mode)
 xplr.config.modes.builtin.switch_layout = {
   name = "switch layout",
+  layout = "HelpMenu",
   key_bindings = {
     on_key = {
       ["1"] = {
@@ -2391,6 +2347,68 @@ xplr.config.modes.builtin.switch_layout = {
   },
 }
 
+-- The builtin vroot mode.
+--
+-- Type: [Mode](https://xplr.dev/en/mode)
+xplr.config.modes.builtin.vroot = {
+  name = "vroot",
+  layout = "HelpMenu",
+  key_bindings = {
+    on_key = {
+      ["v"] = {
+        help = "toggle vroot",
+        messages = {
+          "PopMode",
+          "ToggleVroot",
+        },
+      },
+      ["."] = {
+        help = "vroot $PWD",
+        messages = {
+          "PopMode",
+          {
+            BashExecSilently0 = [===[
+              "$XPLR" -m 'SetVroot: %q' "${PWD:?}"
+            ]===],
+          },
+        },
+      },
+      ["/"] = {
+        help = "vroot /",
+        messages = {
+          "PopMode",
+          { SetVroot = "/" },
+        },
+      },
+      ["~"] = {
+        help = "vroot $HOME",
+        messages = {
+          "PopMode",
+          {
+            BashExecSilently0 = [===[
+              "$XPLR" -m 'SetVroot: %q' "${HOME:?}"
+            ]===],
+          },
+        },
+      },
+      ["ctrl-r"] = {
+        help = "reset vroot",
+        messages = {
+          "PopMode",
+          "ResetVroot",
+        },
+      },
+      ["ctrl-u"] = {
+        help = "unset vroot",
+        messages = {
+          "PopMode",
+          "UnsetVroot",
+        },
+      },
+    },
+  },
+}
+
 -- This is where you define custom modes.
 --
 -- Type: mapping of the following key-value pairs:
@@ -2436,17 +2454,9 @@ xplr.fn.builtin.try_complete_path = function(m)
     return
   end
 
-  local function splitlines(str)
-    local res = {}
-    for s in str:gmatch("[^\r\n]+") do
-      table.insert(res, s)
-    end
-    return res
-  end
-
-  local function matches_all(str, files)
-    for _, p in ipairs(files) do
-      if string.sub(p, 1, #str) ~= str then
+  local function matches_all(str, paths)
+    for _, path in ipairs(paths) do
+      if string.sub(path, 1, #str) ~= str then
         return false
       end
     end
@@ -2454,12 +2464,23 @@ xplr.fn.builtin.try_complete_path = function(m)
   end
 
   local path = m.input_buffer
+  local explorer_config = {
+    filters = {
+      { filter = "RelativePathDoesStartWith", input = xplr.util.basename(path) },
+    },
+  }
+  local parent = xplr.util.dirname(path)
+  if not parent or parent == "" then
+    parent = "./"
+  elseif parent ~= "/" then
+    parent = parent .. "/"
+  end
 
-  local p = assert(io.popen(string.format("ls -d %q* 2>/dev/null", path)))
-  local out = p:read("*all")
-  p:close()
-
-  local found = splitlines(out)
+  local nodes = xplr.util.explore(parent, explorer_config)
+  local found = {}
+  for _, node in ipairs(nodes) do
+    table.insert(found, parent .. node.relative_path)
+  end
   local count = #found
 
   if count == 0 then
@@ -2471,7 +2492,7 @@ xplr.fn.builtin.try_complete_path = function(m)
   else
     local first = found[1]
     while #first > #path and matches_all(path, found) do
-      path = string.sub(found[1], 1, #path + 1)
+      path = string.sub(first, 1, #path + 1)
     end
 
     if matches_all(path, found) then
@@ -2504,13 +2525,17 @@ end
 xplr.fn.builtin.fmt_general_table_row_cols_1 = function(m)
   local r = m.tree .. m.prefix
 
+  local function path_escape(path)
+    return string.gsub(string.gsub(path, "\\", "\\\\"), "\n", "\\n")
+  end
+
   if m.meta.icon == nil then
     r = r .. ""
   else
     r = r .. m.meta.icon .. " "
   end
 
-  r = r .. m.relative_path
+  r = r .. path_escape(m.relative_path)
 
   if m.is_dir then
     r = r .. "/"
@@ -2524,7 +2549,7 @@ xplr.fn.builtin.fmt_general_table_row_cols_1 = function(m)
     if m.is_broken then
       r = r .. "×"
     else
-      r = r .. m.symlink.absolute_path
+      r = r .. path_escape(m.symlink.absolute_path)
 
       if m.symlink.is_dir then
         r = r .. "/"
@@ -2639,3 +2664,50 @@ end
 -- You can also use nested tables such as
 -- `xplr.fn.custom.my_plugin.my_function` to define custom functions.
 xplr.fn.custom = {}
+
+-- ## Hooks -------------------------------------------------------------------
+--
+-- This section of the configuration cannot be overwritten by another config
+-- file or plugin, since this is an optional lua return statement specific to
+-- each config file. It can be used to define things that should be explicit
+-- for reasons like performance concerns, such as hooks.
+--
+-- Plugins should expose the hooks, and require users to subscribe to them
+-- explicitly.
+--
+-- Example:
+--
+-- ```lua
+-- return {
+--   -- Add messages to send when the xplr loads.
+--   -- This is similar to the `--on-load` command-line option.
+--   --
+--   -- Type: list of [Message](https://xplr.dev/en/message#message)s
+--   on_load = {
+--     { LogSuccess = "Configuration successfully loaded!" },
+--     { CallLuaSilently = "custom.some_plugin_with_hooks.on_load" },
+--   },
+--
+--   -- Add messages to send when the directory changes.
+--   --
+--   -- Type: list of [Message](https://xplr.dev/en/message#message)s
+--   on_directory_change = {
+--     { LogSuccess = "Changed directory" },
+--     { CallLuaSilently = "custom.some_plugin_with_hooks.on_directory_change" },
+--   },
+--
+--   -- Add messages to send when the focus changes.
+--   --
+--   -- Type: list of [Message](https://xplr.dev/en/message#message)s
+--   on_focus_change = {
+--     { LogSuccess = "Changed focus" },
+--     { CallLuaSilently = "custom.some_plugin_with_hooks.on_focus_change" },
+--   }
+-- }
+-- ```
+
+return {
+  on_load = {},
+  on_directory_change = {},
+  on_focus_change = {},
+}
